# coding=utf-8
from dateutil.easter import EASTER_WESTERN

from holidata.utils import SmartDayArrow
from .holidays import Holiday, Locale

"""
Public Holidays Act (Act No 36 of 1994).
sources: 
https://www.gov.za/sites/default/files/gcis_document/201409/act36of1994.pdf
https://www.gov.za/sites/default/files/gcis_document/201409/act48of1995.pdf
"""


class en_ZA(Locale):
    """
    12-25: [NF] Christmas Day
    2 days before Easter: [NRV] Good Friday
    1 day after Easter: [NRV] Family Day
    """

    locale = "en-ZA"
    easter_type = EASTER_WESTERN

    def holiday_new_years_day(self):
        """01-01: [NF] New Year's Day"""
        return self.get_holidays(SmartDayArrow(self.year, 1, 1), "New Year's Day")

    def holiday_human_rights_day(self):
        """03-21: [NF] Human Rights Day"""
        return self.get_holidays(SmartDayArrow(self.year, 3, 21), "Human Rights Day")

    def holiday_freedom_day(self):
        """04-27: [NF] Freedom Day"""
        return self.get_holidays(SmartDayArrow(self.year, 4, 27), "Freedom Day")

    def holiday_workers_day(self):
        """05-01: [NF] Worker's Day"""
        return self.get_holidays(SmartDayArrow(self.year, 5, 1), "Worker's Day")

    def holiday_youth_day(self):
        """06-16: [NF] Youth Day"""
        return self.get_holidays(SmartDayArrow(self.year, 6, 16), "Youth Day")

    def holiday_national_womens_day(self):
        """08-09: [NF] National Women's Day"""
        return self.get_holidays(SmartDayArrow(self.year, 8, 9), "National Women's Day")

    def holiday_heritage_day(self):
        """09-24: [NF] Heritage Day"""
        return self.get_holidays(SmartDayArrow(self.year, 9, 24), "Heritage Day")

    def holiday_day_of_reconciliation(self):
        """12-16: [NF] Day of Reconciliation"""
        return self.get_holidays(SmartDayArrow(self.year, 12, 16), "Day of Reconciliation")

    def holiday_day_of_goodwill(self):
        """12-26: [NF] Day of Goodwill"""
        return self.get_holidays(SmartDayArrow(self.year, 12, 26), "Day of Goodwill")

    def get_holidays(self, original_date, description):
        """
        Applies section 2.1 of the Public Holidays Act (Act No 36 of 1994):
        'Whenever any public holiday falls on a Sunday, the following Monday shall be a public holiday.'
        """
        if original_date.weekday() == "sunday":
            supplement_date = original_date.shift(days=1)
            return [
                Holiday(
                    locale=self.locale,
                    region='',
                    date=original_date,
                    description=description,
                    flags="NF",
                    notes=""),
                Holiday(
                    locale=self.locale,
                    region='',
                    date=supplement_date,
                    description=description + " (Supplement)",
                    flags="NF",
                    notes="Supplement holiday")
            ]
        else:
            return [Holiday(
                locale=self.locale,
                region='',
                date=original_date,
                description=description,
                flags="NF",
                notes="")]



from .holidays import Country


class CH(Country):
    id = "CH"
    languages = ["de"]
    default_lang = "de"
    regions = ["AG", "AI", "AR", "BE", "BL", "BS", "FR", "GE", "GL", "GR", "JU", "LU", "NE", "NW", "OW", "SG", "SH",
               "SO", "SZ", "TI", "TG", "UR", "VD", "VS", "ZG", "ZH"]



# coding=utf-8
from dateutil.easter import EASTER_ORTHODOX

from .holidays import Locale


class ru_RU(Locale):
    """
    01-01: [NF] Новый Год
    01-07: [NRF] Рождество Христово
    02-23: [NF] День защитника Отечества
    03-08: [NF] Международный женский день
    05-01: [NF] Праздник весны и труда
    05-09: [NF] День Победы
    06-12: [NF] День России
    11-04: [NF] День народного единства
    Easter: [NRV] Пасха
    """

    locale = "ru-RU"
    easter_type = EASTER_ORTHODOX



# coding=utf-8
from dateutil.easter import EASTER_WESTERN

from .holidays import Locale


class de_BE(Locale):
    """
    01-01: [NF] Neujahr
    05-01: [NF] Tag der Arbeit
    07-21: [NF] Nationalfeiertag
    08-15: [NRF] Mariä Himmelfahrt
    11-01: [NRF] Allerheiligen
    11-11: [NF] Waffenstillstand
    12-25: [NRF] Weihnacht
    Easter: [NRV] Ostern
    1 day after Easter: [NRV] Ostermontag
    39 days after Easter: [NRV] Christi Himmelfahrt
    49 days after Easter: [NRV] Pfingsten
    50 days after Easter: [NRV] Pfingstmontag
    """

    locale = "de-BE"
    easter_type = EASTER_WESTERN



from .holidays import Country


class DE(Country):
    id = "DE"
    languages = ["de"]
    default_lang = "de"
    regions = ["BB", "BE", "BH", "BW", "BY", "HE", "HH", "MV", "NI", "NW", "RP", "SH", "SL", "SN", "ST", "TH"]



import re

import pytest

from holidata import Locale, Country
from tests import HOLIDATA_YEAR_MAX


@pytest.fixture(params=range(2011, HOLIDATA_YEAR_MAX))
def year(request):
    return request.param


@pytest.fixture(params=Locale.plugins)
def locale(request, year):
    return request.param(year)


@pytest.fixture()
def holidays(locale):
    return locale.holidays


@pytest.fixture(params=Country.plugins)
def country(request):
    return request.param()


def test_country_should_be_constructable(country):
    pass


def test_holiday_should_not_be_of_type_national_if_region_defined(holidays):
    for holiday in holidays:
        if holiday.region == '':
            assert "N" in holiday.flags, "Holiday '{}' ({}) in locale {} must have flag 'N': it has no regions defined".format(holiday.description, holiday.date.strftime("%Y-%m-%d"), holiday.locale)


def test_holiday_should_be_of_type_national_if_no_region_defined(holidays):
    for holiday in holidays:
        if holiday.region != '':
            assert "N" not in holiday.flags, "Holiday '{}' ({}) in locale {} must not have flag 'N': it has regions defined".format(holiday.description, holiday.date.strftime("%Y-%m-%d"), holiday.locale)


def test_holiday_should_be_of_type_either_fixed_or_variable(holidays):
    for holiday in holidays:
        date_is_fixed = "F" in holiday.flags
        date_is_variable = "V" in holiday.flags

        assert not (date_is_variable and date_is_fixed), "Holiday '{}' ({}) in locale {} must not have both flags 'F' and 'V'".format(holiday.description, holiday.date.strftime("%Y-%m-%d"), holiday.locale)
        assert (date_is_variable or date_is_fixed), "Holiday '{}' ({}) in locale {} must have either flag 'F' or 'V'".format(holiday.description, holiday.date.strftime("%Y-%m-%d"), holiday.locale)


def test_holiday_flags_should_be_in_the_correct_order(holidays):
    for holiday in holidays:
        match = re.search(r"^N?R?[FV]?$", "{}".format(holiday.flags))

        assert match is not None, "Flags for holiday '{}' ({}) in locale {} are not in the correct order. Flags '{}' should match 'N?R?[FV]?'".format(holiday.description, holiday.date.strftime("%Y-%m-%d"), holiday.locale, holiday.flags)



# coding=utf-8
from dateutil.easter import EASTER_WESTERN

from .holidays import Locale


class nb_NO(Locale):
    """
    01-01: [NF] Nyttårsdag
    05-01: [NF] Offentlig Høytidsdag
    05-08: [NF] Frigjøringsdag 1945
    05-17: [NF] Grunnlovsdag
    12-24: [NRF] Julaften
    12-25: [NRF] Juledag
    12-26: [NRF] Juledag
    12-31: [NF] Nyttårsaften
    49 days before Easter: [NRV] Fastelavn
    7 days before Easter: [NRV] Palmesøndag
    3 days before Easter: [NRV] Skjærtorsdag
    2 days before Easter: [NRV] Langfredag
    Easter: [NRV] Påskedag
    1 day after Easter: [NRV] Påskedag
    39 days after Easter: [NRV] Kristi Himmelfartsdag
    49 days after Easter: [NRV] Pinsedag
    50 days after Easter: [NRV] Pinsedag
    """

    locale = "nb-NO"
    easter_type = EASTER_WESTERN



# coding=utf-8
from dateutil.easter import EASTER_WESTERN

from .holidays import Locale


# Information gathered from Brazilian legislation.

# National holidays:
# Law 10.607/2002: http://www.planalto.gov.br/ccivil_03/leis/2002/l10607.htm
# Law 6.802/1980:  http://www.planalto.gov.br/ccivil_03/leis/l6802.htm

# State holidays
# AC: State laws 1.538/2004, 1.411/2001, 14/1964, 1.526/2004, and 57/1965
# AL: State laws 5.508/1993, 5.509/1993, 5.724/1995, and Decree 68.782 (30/dec/2019)
# AP: State law 667/2002 and Art. 355 of the Constitution of the Federative Republic of Brazil
# AM: State laws 25/1977 and 84/2010
# BA: Art. 6 of the Constitution of the Federative Republic of Brazil
# CE: State laws 9.093/1995, 9.093/1995 and Art. 18 of the Constitution of the State of Ceará
# DF: District law 963/1995
# ES: State law 11.010/2019
# GO: State law 20.756/2020
# MA: State law 2.457/1964
# MT: State law 7.879/2002
# MS: State law 10/1979
# MG: Art. 256 of the Constitution of the State of Minas Gerais
# PA: State law 5.999/1996
# PB: State law 3.489/1967
# PR: State law 4.658/1962
# PE: State law 13.835/2009
# PI: State law 176/1937
# RJ: State laws 5.243/2008, 5.198/2008, and 4.007/2002
# RN: State laws 7.831/2000 and 8.913/2006
# RS: Art. 6 of the Constitution of the State of Rio Grande do Sul and Decree 36.180 (18/set/1995)
# RO: State laws 2.291/2010 and 1.026/2001
# RR: Art. 9 of the Constitution of the State of Roraima
# SC: State laws 12.906/2004 and 10.306/1996
# SP: State law 9.497/1997
# SE: Art. 269 of the Constitution of the State of Sergipe
# TO: State laws 98/1989, 960/1998, 627/1993


class pt_BR(Locale):
    """
    01-01: [NF] Confraternização Universal
    01-04: [RO] [F] Criação do Estado de Rondônia
    01-23: [AC] [RF] Dia do Evangélico no Acre
    03-06: [PE] [F] Revolução Pernambucana de 1817
    03-08: [AC] [F] Dia Internacional da Mulher
    03-18: [TO] [F] Autonomia do Estado de Tocantins
    03-19: [AP, CE] [RF] Dia de São José
    03-25: [CE] [F] Abolição da Escravidão no Ceará
    04-21: [DF] [F] Fundação de Brasília
    04-21: [MG] [F] Execução de Tiradentes
    04-21: [NF] Tiradentes
    04-23: [RJ] [RF] Dia de São Jorge
    05-01: [NF] Dia Internacional do Trabalhador
    06-15: [AC] [F] Aniversário do Estado do Acre
    06-18: [RO] [RF] Dia do Evangélico em Rondônia
    06-24: [AL, PE] [RF] São João
    06-29: [AL] [RF] São Pedro
    07-02: [BA] [F] Independência da Bahia
    07-08: [SE] [F] Emancipação Política de Sergipe
    07-09: [SP] [F] Revolução Constitucionalista de 1932
    07-26: [GO] [F] Fundação da Cidade de Goiás
    07-28: [MA] [F] Adesão do Maranhão à Independência do Brasil
    08-05: [PB] [F] Fundação do Estado da Paraíba
    08-07: [RN] [F] Dia do Rio Grande do Norte
    08-11: [SC] [F] Dia de Santa Catarina
    08-15: [CE] [RF] Dia de Nossa Senhora da Assunção
    08-15: [PA] [F] Adesão do Pará à Independência do Brasil
    09-05: [AC] [F] Dia da Amazônia
    09-05: [AM] [F] Elevação do Amazonas à Categoria de Província
    09-07: [NF] Independência do Brasil
    09-08: [TO] [F] Nossa Senhora da Natividade
    09-13: [AP] [F] Criação do Território Federal do Amapá
    09-16: [AL] [F] Emancipação Política do Alagoas
    09-20: [RS] [F] Dia do Gaúcho
    10-03: [RN] [F] Mártires de Cunhaú e Uruaçu
    10-05: [RR] [F] Criação dos Estado de Roraima
    10-05: [TO] [F] Criação dos Estado de Tocantins
    10-11: [MS] [F] Criação do Estado do Mato Grosso do Sul
    10-12: [NRF] Nossa Senhora Aparecida
    10-19: [PI] [F] Dia do Piauí
    10-24: [GO] [F] Pedra Fundamental de Goiânia
    11-02: [NRF] Finados
    11-15: [NF] Proclamação da República
    11-17: [AC] [F] Assinatura do Tratado de Petrópolis
    11-20: [AL] [F] Morte de Zumbi dos Palmares
    11-20: [AM, MT, RJ] [F] Dia da Consciência Negra
    11-25: [SC] [RF] Dia de Santa Catarina de Alexandria
    11-30: [DF] [RF] Dia do Evangélico do Distrito Federal
    12-08: [AM] [RF] Nossa Senhora da Conceição
    12-19: [PR] [F] Emancipação Política do Estado do Paraná
    12-25: [NRF] Natal
    47 days before Easter: [NRV] Carnaval
    Easter: [NRV] Páscoa
    """

    locale = "pt-BR"
    easter_type = EASTER_WESTERN



# coding=utf-8
from dateutil.easter import EASTER_WESTERN

from .holidays import Locale


class it_IT(Locale):
    """
    01-01: [NF] Capodanno
    01-06: [NRF] Epifania
    04-25: [NF] Festa della liberazione
    05-01: [NF] Festa del lavoro
    06-02: [NF] Festa della repubblica
    08-15: [NRF] Assunzione (ferragosto)
    11-01: [NRF] Ognissanti
    12-08: [NRF] Immacolata concezione
    12-25: [NRF] Natale
    12-26: [NRF] S.to Stefano
    Easter: [NRV] Pasqua
    1 day after Easter: [NRV] Pasquetta
    """

    locale = "it-IT"
    easter_type = EASTER_WESTERN



# coding=utf-8
from dateutil.easter import EASTER_WESTERN

from .holidays import Locale


class de_AT(Locale):
    """
    01-01: [NF] Neujahr
    01-06: [NRF] Heilige drei Könige
    03-19: [2,6,7,8] [RF] Josef
    05-01: [NF] Staatsfeiertag
    05-04: [4] [F] Florian
    08-15: [NRF] Mariä Himmelfahrt
    09-24: [5] [F] Rupert
    10-10: [2] [F] Tag der Volksabstimmung
    10-26: [NF] Nationalfeiertag
    11-01: [NRF] Allerheiligen
    11-11: [1] [F] Martin
    11-15: [9,3] [F] Leopold
    12-08: [NRF] Mariä Empfängnis
    12-24: [NRF] Heiliger Abend
    12-25: [NRF] Christtag
    12-26: [NF] Stefanitag
    12-31: [NF] Silvester
    2 days before Easter: [NRV] Karfreitag
    Easter: [NRV] Ostersonntag
    1 day after Easter: [NRV] Ostermontag
    39 days after Easter: [NRV] Christi Himmelfahrt
    49 days after Easter: [NRV] Pfingstsonntag
    50 days after Easter: [NRV] Pfingstmontag
    60 days after Easter: [NRV] Fronleichnam
    """

    locale = "de-AT"
    easter_type = EASTER_WESTERN



"""
Provides date-handling related utils.
"""
import dateutil
from arrow import Arrow


class SmartDayArrow(Arrow):
    """
    A wrapper around Arrow datetime reference that provides additional
    convenience methods.
    """

    def weekday(self):
        """
        Provide a more readable weekday representation.
        """

        weekdays = [
            "monday",
            "tuesday",
            "wednesday",
            "thursday",
            "friday",
            "saturday",
            "sunday",
        ]

        return weekdays[Arrow.weekday(self)]

    def shift_to_weekday(self, day, order=1, reverse=False, including=False):
        """
        Shifts to {order}. weekday in the given direction, i.e.
        2. monday before this date would be:

        >>> arrow.shift_to_weekday("monday", order=2, reverse=True)
        """

        result = self

        if including and result.weekday() == day:
            if order == 1:
                return result
            else:
                order = order - 1

        while order > 0:
            result = result.shift(days=1 if not reverse else -1)
            if day == result.weekday():
                order = order - 1

        return result


def easter(year, easter_type):
    date = dateutil.easter.easter(year, easter_type)
    return SmartDayArrow(date.year, date.month, date.day)


def month_reference(year, month, first=True):
    months = [
        "january",
        "february",
        "march",
        "april",
        "may",
        "june",
        "july",
        "august",
        "september",
        "october",
        "november",
        "december",
    ]

    month = months.index(month.lower()) + 1

    if first:
        return SmartDayArrow(year, month, 1)
    else:
        return SmartDayArrow(
            year if month != 12 else year+1,
            month+1 if month != 12 else 1,
            1).shift(days=-1)



# coding=utf-8
from dateutil.easter import EASTER_WESTERN

from .holidays import Locale

"""
sources
https://www.riigiteataja.ee/akt/109032011007 (Public Holidays and Days of National Importance Act)
"""


class et_EE(Locale):
    """
    01-01: [NF] Uusaasta
    02-24: [NF] Iseseisvuspäev, Eesti Vabariigi aastapäev
    05-01: [NF] Kevadpüha
    06-23: [NF] Võidupüha
    06-24: [NF] Jaanipäev
    08-20: [NF] Taasiseseisvumispäev
    12-24: [NF] Jõululaupäev
    12-25: [NF] Esimene jõulupüha
    12-26: [NF] Teine jõulupüha
    2 days before Easter: [NRV] Suur reede
    Easter: [NRV] Ülestõusmispühade 1. püha
    49 days after Easter: [NRV] Nelipühade 1. püha
    """

    locale = "et-EE"
    easter_type = EASTER_WESTERN



import pytest
from snapshottest.file import FileSnapshot
from snapshottest.formatter import Formatter

from holidata import Locale
from tests import HOLIDATA_YEAR_MAX

SNAPSHOT_FILE_PATH_PATTERN = "snapshots/snap_test_holidata/{}[{}-{}] 1.py"


@pytest.fixture(params=range(2011, HOLIDATA_YEAR_MAX))
def year(request):
    return request.param


@pytest.fixture(params=Locale.plugins)
def locale(request, year):
    return request.param(year)


def test_holidata_produces_holidays_for_locale_and_year(snapshot, tmpdir, locale):
    temp_file = tmpdir.join("{}.{}.py".format(locale.locale, locale.year))

    export_data = [h.as_dict() for h in locale.holidays]
    export_data.sort(key=lambda x: x["date"])
    temp_file.write(Formatter().format(export_data, 0))

    try:
        snapshot.assert_match(FileSnapshot(str(temp_file)))
    except AssertionError:
        with open(temp_file, "r") as tf:
            actual = "".join(tf.readlines())

        snapshot_file = SNAPSHOT_FILE_PATH_PATTERN.format(
            "test_holidata_produces_holidays_for_locale_and_year",
            locale.__class__.__name__,
            locale.year)

        with open(snapshot_file) as sf:
            expected = "".join(sf.readlines())

        assert (actual == expected)



# coding=utf-8
from dateutil.easter import EASTER_WESTERN

from holidata.utils import SmartDayArrow, easter
from .holidays import Locale, Holiday

"""
Information taken from government websites around 2020-06
    https://administracion.gob.es/pag_Home/atencionCiudadana/calendarios/laboral.html
    http://www.seg-social.es/wps/portal/wss/internet/CalendarioLaboral

    2011: https://www.boe.es/eli/es/res/2010/10/07/(1)
          https://www.boe.es/eli/es/res/2010/11/24/(1)
    2012: https://www.boe.es/eli/es/res/2011/10/06/(1)
    2013: https://www.boe.es/eli/es/res/2012/10/30/(1)
          https://www.boe.es/eli/es/res/2012/11/12/(2)
    2014: https://www.boe.es/eli/es/res/2013/11/08/(3)
          https://www.boe.es/eli/es/res/2013/11/21/(1)
    2015: https://www.boe.es/eli/es/res/2014/10/17/(3)
    2016: https://www.boe.es/eli/es/res/2015/10/19/(1)
    2017: https://www.boe.es/eli/es/res/2016/10/04/(1)
    2018: https://www.boe.es/eli/es/res/2017/10/09/(1)
          https://www.boe.es/eli/es/res/2017/10/09/(1)/corrigendum/20171019
          https://www.boe.es/eli/es/res/2017/10/09/(1)/corrigendum/20171025
    2019: https://www.boe.es/eli/es/res/2018/10/16/(1)
    2020: https://www.boe.es/eli/es/res/2019/10/03/(1)
    2021: https://www.boe.es/eli/es/res/2020/10/28/(1)
    2022: https://www.boe.es/eli/es/res/2021/10/14/(3)
    2023: https://www.boe.es/eli/es/res/2022/10/07/(2)

Regional governments
    [AN] https://www.juntadeandalucia.es/temas/trabajar/relaciones/calendario.html

Also those sites for some information
    https://es.wikipedia.org/wiki/Calendario_laboral
"""


class es_ES(Locale):
    """
    01-01: [NF] Año Nuevo
    01-06: [NRF] Epifanía del Señor
    05-01: [NF] Fiesta del Trabajo
    08-15: [NRF] Asunción de la Virgen
    10-12: [NF] Fiesta Nacional de España
    11-01: [NRF] Todos los Santos
    12-06: [NF] Día de la Constitución Española
    12-08: [NRF] Inmaculada Concepción
    12-25: [NRF] Natividad del Señor
    2 days before Easter: [NRV] Viernes Santo
    Easter: [NRV] Pascua
    """

    locale = "es-ES"
    easter_type = EASTER_WESTERN

    def holiday_lunes_siguiente_al_ano_nuevo(self):
        if self.year == 2012:
            regions = [""]
        elif self.year == 2017:
            regions = ["AN", "AR", "AS", "CL", "MC", "ML"]
        elif self.year == 2023:
            regions = ["AN", "AR", "AS", "CL", "MC"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            SmartDayArrow(self.year, 1, 1).shift_to_weekday("monday", including=True),
            "Lunes siguiente al Año Nuevo",
            "NF" if regions == [""] else "F"
        ) for region in regions]

    def holiday_lunes_siguiente_a_la_epifania_del_senor(self):
        if self.year == 2013:
            regions = ["AN", "AR", "AS", "CB", "CE", "CL", "CM", "CN", "EX", "MC", "MD", "ML", "NC"]
        elif self.year == 2019:
            regions = ["AN", "AR", "AS", "CE", "CL", "CN", "EX", "MC", "MD", "ML", "NC"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            SmartDayArrow(self.year, 1, 6).shift_to_weekday("monday", including=True),
            "Lunes siguiente a la Epifanía del Señor",
            "NRF" if regions == [""] else "RF"
        ) for region in regions]

    def holiday_martes_de_carnaval(self):
        if self.year == 2023:
            return [Holiday(
                self.locale,
                "EX",
                SmartDayArrow(self.year, 2, 21),
                "Martes de Carnaval",
                "V"
            )]
        else:
            return []

    def holiday_dia_de_andalucia(self):
        if self.year == 2016:
            date = SmartDayArrow(self.year, 2, 29)
        elif self.year == 2021:
            date = SmartDayArrow(self.year, 3, 1)
        else:
            date = SmartDayArrow(self.year, 2, 28)

        return [Holiday(
            self.locale,
            "AN",
            date,
            "Día de Andalucía",
            "F"
        )]

    def holiday_dia_de_las_illes_balears(self):
        if self.year in [2011, 2012, 2013, 2014, 2016, 2017, 2018, 2019, 2021, 2022, 2023]:
            return [Holiday(
                self.locale,
                "IB",
                SmartDayArrow(self.year, 3, 1),
                "Día de las Illes Balears",
                "F"
            )]
        else:
            return []

    def holiday_estatuto_de_autonomia_de_la_ciudad_de_melilla(self):
        if self.year in [2020, 2021]:
            return [Holiday(
                self.locale,
                "ML",
                SmartDayArrow(self.year, 3, 13),
                "Estatuto de Autonomía de la Ciudad de Melilla",
                "F"
            )]
        else:
            return []

    def holiday_san_jose(self):
        if self.year == 2011:
            regions = ["CM", "GA", "MC", "ML", "VC"]
        elif self.year == 2012:
            regions = ["CL", "MC", "MD", "ML", "NC", "RI", "VC"]
        elif self.year == 2013:
            regions = ["MC", "ML", "VC"]
        elif self.year == 2014:
            regions = ["MC", "ML", "NC", "VC"]
        elif self.year == 2015:
            regions = ["MC", "MD", "ML", "NC", "PV", "VC"]
        elif self.year in [2016]:
            regions = ["MC", "ML", "VC"]
        elif self.year == 2018:
            regions = ["MC", "VC"]
        elif self.year == 2019:
            regions = ["GA", "MC", "NC", "PV", "VC"]
        elif self.year == 2020:
            regions = ["CM", "GA", "MC", "NC", "PV", "VC"]
        elif self.year == 2021:
            regions = ["EX", "GA", "MC", "MD", "NC", "PV", "VC"]
        elif self.year == 2022:
            regions = ["VC"]
        elif self.year == 2023:
            regions = ["MD"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            SmartDayArrow(self.year, 3, 19),
            "San José",
            "NRF" if regions == [""] else "RF"
        ) for region in regions]

    def holiday_lunes_de_fallas(self):
        if self.year == 2013:
            return [Holiday(
                self.locale,
                "VC",
                SmartDayArrow(self.year, 3, 18),
                "Lunes de Fallas",
                "F"
            )]
        else:
            return []

    def holiday_traslado_de_san_jose(self):
        if self.year == 2013:
            return [Holiday(
                self.locale,
                "MD",
                SmartDayArrow(self.year, 3, 18),
                "Traslado de San José",
                "F"
            )]
        else:
            return []

    def holiday_dia_siguente_a_san_jose(self):
        if self.year == 2015:
            return [Holiday(
                self.locale,
                "GA",
                SmartDayArrow(self.year, 3, 20),
                "Día siguiente a San José",
                "RF"
            )]
        else:
            return []

    def holiday_lunes_siguiente_a_san_jose(self):
        if self.year in [2017]:
            regions = ["EX", "MD"]
        elif self.year == 2023:
            regions = ["MD"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            SmartDayArrow(self.year, 3, 19).shift_to_weekday("monday", True),
            "Lunes siguiente a San José",
            "NRF" if regions == [""] else "RF"
        ) for region in regions]

    def holiday_san_jorge__dia_de_aragon(self):
        if self.year == 2017:
            date = SmartDayArrow(self.year, 4, 24)
        else:
            date = SmartDayArrow(self.year, 4, 23)

        return [Holiday(
            self.locale,
            "AR",
            date,
            "San Jorge / Día de Aragón",
            "RF"
        )]

    def holiday_lunes_siguiente_a_san_jorge__dia_de_aragon(self):
        if self.year == 2023:
            date = SmartDayArrow(self.year, 4, 24)
        else:
            return []

        return [Holiday(
            self.locale,
            "AR",
            date,
            "Lunes siguiente a San Jorge / Día de Aragón",
            "RF"
        )]

    def holiday_fiesta_de_castilla_y_leon(self):
        if self.year == 2017:
            date = SmartDayArrow(self.year, 4, 24)
        else:
            date = SmartDayArrow(self.year, 4, 23)

        return [Holiday(
            self.locale,
            "CL",
            date,
            "Fiesta de Castilla y León",
            "F"
        )]

    def holiday_lunes_siguiente_a_la_fiesta_del_trabajo(self):
        if self.year == 2011:
            regions = ["AN", "AR", "AS", "CB", "CE", "EX", "MC", "VC"]
        elif self.year == 2016:
            regions = ["AN", "AR", "AS", "CL", "CN", "EX", "MD"]
        elif self.year == 2022:
            regions = ["AN", "AR", "AS", "CL", "EX", "MC"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            SmartDayArrow(self.year, 5, 1).shift_to_weekday("monday", including=True),
            "Lunes siguiente a la Fiesta del Trabajo",
            "NF" if regions == [""] else "F"
        ) for region in regions]

    def holiday_fiesta_de_la_comunidad_de_madrid(self):
        if self.year in [2016, 2021]:
            return []
        else:
            return [Holiday(
                self.locale,
                "MD",
                SmartDayArrow(self.year, 5, 2),
                "Fiesta de la Comunidad de Madrid",
                "F"
            )]

    def holiday_lunes_siguiente_al_dia_de_la_comunidad_de_madrid(self):
        if self.year in [2021]:
            return [Holiday(
                self.locale,
                "MD",
                SmartDayArrow(self.year, 5, 2).shift_to_weekday("monday", including=True),
                "Lunes siguiente al Día de la Comunidad de Madrid",
                "F"
            )]
        else:
            return []

    def holiday_dia_de_las_letras_gallegas(self):
        if self.year in [2015, 2020]:
            return []
        else:
            return [Holiday(
                self.locale,
                "GA",
                SmartDayArrow(self.year, 5, 17),
                "Día de las Letras Gallegas",
                "F"
            )]

    def holiday_dia_de_canarias(self):
        if self.year in [2021]:
            return []
        else:
            return [Holiday(
                self.locale,
                "CN",
                SmartDayArrow(self.year, 5, 30),
                "Día de Canarias",
                "F"
            )]

    def holiday_dia_de_castilla_la_mancha(self):
        if self.year in [2014, 2015, 2020]:
            return []
        else:
            return [Holiday(
                self.locale,
                "CM",
                SmartDayArrow(self.year, 5, 31),
                "Día de Castilla-La Mancha",
                "F"
            )]

    def holiday_dia_de_la_region_de_murcia(self):
        if self.year == 2013:
            return []
        elif self.year == 2019:
            date = SmartDayArrow(self.year, 6, 10)
        else:
            date = SmartDayArrow(self.year, 6, 9)

        return [Holiday(
            self.locale,
            "MC",
            date,
            "Día de la Región de Murcia",
            "F"
        )]

    def holiday_dia_de_la_rioja(self):
        if self.year in [2013, 2019]:
            date = SmartDayArrow(self.year, 6, 10)
        else:
            date = SmartDayArrow(self.year, 6, 9)

        if date is not None:
            return [Holiday(
                self.locale,
                "RI",
                date,
                "Día de La Rioja",
                "F"
            )]
        else:
            return []

    def holiday_san_juan(self):
        if self.year == 2011:
            regions = ["CT"]
        elif self.year == 2013:
            regions = ["CT", "GA"]
        elif self.year == 2014:
            regions = ["CT"]
        elif self.year == 2015:
            regions = ["CT"]
        elif self.year == 2016:
            regions = ["CT", "GA"]
        elif self.year == 2017:
            regions = ["CT"]
        elif self.year == 2019:
            regions = ["CT", "VC"]
        elif self.year == 2020:
            regions = ["CT", "GA", "VC"]
        elif self.year == 2021:
            regions = ["CT", "VC"]
        elif self.year == 2022:
            regions = ["CT", "GA", "VC"]
        elif self.year == 2023:
            regions = ["CT", "VC"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            SmartDayArrow(self.year, 6, 24),
            "San Juan",
            "NRF" if regions == [""] else "RF"
        ) for region in regions]

    def holiday_santiago_apostol__dia_nacional_de_galicia(self):
        if self.year == 2021:
            return []
        elif self.year == 2023:
            regions = ["CL", "GA", "NC", "PV"]
        else:
            regions = ["GA"]

        return [Holiday(
            self.locale,
            region,
            SmartDayArrow(self.year, 7, 25),
            "Santiago Apóstol / Día Nacional de Galicia",
            "RF"
        ) for region in regions]

    def holiday_santiago_apostol(self):
        if self.year == 2011:
            regions = ["CL", "MD", "NC", "PV", "RI"]
        elif self.year == 2012:
            regions = ["CB"]
        elif self.year == 2013:
            regions = ["CB", "NC", "PV"]
        elif self.year == 2014:
            regions = ["CB"]
        elif self.year == 2015:
            regions = ["NC", "PV"]
        elif self.year == 2016:
            regions = ["MD", "NC", "PV", "RI"]
        elif self.year == 2017:
            regions = ["NC", "PV"]
        elif self.year == 2019:
            regions = ["CB", "PV"]
        elif self.year == 2020:
            regions = ["PV"]
        elif self.year == 2022:
            regions = ["MD", "NC", "PV"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            SmartDayArrow(self.year, 7, 25),
            "Santiago Apóstol",
            "NRF" if regions == [""] else "RF"
        ) for region in regions]

    def holiday_dia_de_las_instituciones_de_cantabria(self):
        if self.year in [2011, 2016, 2017, 2018, 2020, 2021, 2022, 2023]:
            return [Holiday(
                self.locale,
                "CB",
                SmartDayArrow(self.year, 7, 28),
                "Día de las Instituciones de Cantabria",
                "F"
            )]
        else:
            return []

    def holiday_nuestra_senora_de_africa(self):
        if self.year in [2022, 2023]:
            return [Holiday(
                self.locale,
                "CE",
                SmartDayArrow(self.year, 8, 5),
                "Nuestra Señora de África",
                "RF"
            )]
        else:
            return []

    def holiday_lunes_siguiente_a_la_asuncion_de_la_virgen(self):
        if self.year == 2021:
            regions = ["AN", "AR", "AS", "CL", "CN"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            SmartDayArrow(self.year, 8, 15).shift_to_weekday("monday", including=True),
            "Lunes siguiente a la Asunción de la Virgen",
            "RF"
        ) for region in regions]

    def holiday_dia_de_ceuta(self):
        if self.year in [2016, 2017, 2019, 2020, 2021, 2022, 2023]:
            return [Holiday(
                self.locale,
                "CE",
                SmartDayArrow(self.year, 9, 2),
                "Día de Ceuta",
                "F"
            )]
        else:
            return []

    def holiday_v_centenario_vuelta_al_mundo(self):
        if self.year == 2022:
            return [Holiday(
                self.locale,
                "PV",
                SmartDayArrow(self.year, 9, 6),
                "V Centenario Vuelta al Mundo",
                "F"
            )]
        else:
            return []

    def holiday_dia_de_asturias(self):
        if self.year in [2019]:
            return []
        else:
            return [Holiday(
                self.locale,
                "AS",
                SmartDayArrow(self.year, 9, 8),
                "Día de Asturias",
                "F"
            )]

    def holiday_lunes_siguiente_al_dia_de_asturias(self):
        if self.year in [2013, 2019]:
            return [Holiday(
                self.locale,
                "AS",
                SmartDayArrow(self.year, 9, 8).shift_to_weekday("monday", including=True),
                "Lunes siguiente al Día de Asturias",
                "F"
            )]
        else:
            return []

    def holiday_dia_de_extremadura(self):
        if self.year in [2019]:
            return []
        else:
            return [Holiday(
                self.locale,
                "EX",
                SmartDayArrow(self.year, 9, 8),
                "Día de Extremadura",
                "F"
            )]

    def holiday_lunes_siguiente_al_dia_de_extremadura(self):
        if self.year in [2013, 2019]:
            return [Holiday(
                self.locale,
                "EX",
                SmartDayArrow(self.year, 9, 8).shift_to_weekday("monday", including=True),
                "Lunes siguiente al Día de Extremadura",
                "F"
            )]
        else:
            return []

    def holiday_fiesta_nacional_de_cataluna(self):
        if self.year in [2011, 2016, 2022]:
            return []
        else:
            return [Holiday(
                self.locale,
                "CT",
                SmartDayArrow(self.year, 9, 11),
                "Fiesta Nacional de Cataluña",
                "F"
            )]

    def holiday_la_bien_aparecida(self):
        if self.year in [2011, 2012, 2014, 2014, 2015, 2016, 2017, 2018, 2020, 2021, 2022, 2023]:
            return [Holiday(
                self.locale,
                "CB",
                SmartDayArrow(self.year, 9, 15),
                "La Bien Aparecida",
                "RF")]
        else:
            return []

    def holiday_80_aniversario_del_primer_gobierno_vasco(self):
        if self.year in [2016]:
            return [Holiday(
                self.locale,
                "PV",
                SmartDayArrow(self.year, 10, 7),
                "80º aniversario del primer Gobierno Vasco",
                "F"
            )]
        else:
            return []

    def holiday_dia_de_la_comunitat_valenciana(self):
        if self.year in [2011, 2016, 2022]:
            return []
        else:
            return [Holiday(
                self.locale,
                "VC",
                SmartDayArrow(self.year, 10, 9),
                "Día de la Comunitat Valenciana",
                "F"
            )]

    def holiday_lunes_siguiente_a_la_fiesta_nacional_de_espana(self):
        if self.year == 2014:
            regions = ["AN", "AR", "AS", "CE", "CL", "EX"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            SmartDayArrow(self.year, 10, 12).shift_to_weekday("monday", including=True),
            "Lunes siguiente a la Fiesta Nacional de España",
            "F"
        ) for region in regions]

    def holiday_dia_del_pais_vasco_euskadiko_eguna(self):
        if self.year in [2011, 2012, 2013, 2014]:
            return [Holiday(
                self.locale,
                "PV",
                SmartDayArrow(self.year, 10, 25),
                "Día del País Vasco-Euskadiko Eguna",
                "F"
            )]
        else:
            return []

    def holiday_lunes_siguiente_a_todos_los_santos(self):
        if self.year == 2015:
            regions = ["AN", "AR", "AS", "CB", "CE", "CL", "CN", "EX", "GA", "IB"]
        elif self.year == 2020:
            regions = ["AN", "AR", "AS", "CL", "EX", "MD"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            SmartDayArrow(self.year, 11, 1).shift_to_weekday("monday", including=True),
            "Lunes siguiente a Todos los Santos",
            "F"
        ) for region in regions]

    def holiday_lunes_siguiente_al_dia_de_la_constitucion_espanola(self):
        if self.year == 2015:
            regions = ["AN", "AR", "AS", "CE", "CL", "CM", "EX", "IB", "MC", "ML", "RI", "VC"]
        elif self.year == 2020:
            regions = ["AN", "AR", "AS", "CE", "CL", "CN", "EX", "IB", "MC", "MD", "ML", "NC", "RI"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            SmartDayArrow(self.year, 12, 6).shift_to_weekday("monday", including=True),
            "Lunes siguiente al Día de la Constitución Española",
            "F"
        ) for region in regions]

    def holiday_lunes_siguiente_a_la_inmaculada_concepcion(self):
        if self.year == 2013:
            regions = ["AN", "AR", "AS", "CE", "CL", "EX", "MC", "RI"]
        elif self.year == 2019:
            regions = ["AN", "AR", "AS", "CB", "CL", "EX", "MD", "ML", "RI"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            SmartDayArrow(self.year, 12, 8).shift_to_weekday("monday", including=True),
            "Lunes siguiente a La Inmaculada Concepción",
            "RF"
        ) for region in regions]

    def holiday_lunes_siguiente_a_la_natividad_del_senor(self):
        if self.year == 2022:
            regions = ["AN", "AR", "AS", "CB", "CL", "CM", "CN", "EX", "IB", "MC", "MD", "ML", "NC", "RI"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            SmartDayArrow(self.year, 12, 26),
            "Lunes siguiente a la Natividad del Señor",
            "RF"
        ) for region in regions]

    def holiday_san_esteban(self):
        if self.year == 2011:
            regions = ["AN", "AR", "AS", "CE", "CL", "CN", "CT", "EX", "IB", "ML", "NC"]
        elif self.year == 2012:
            regions = ["CT"]
        elif self.year == 2013:
            regions = ["CT", "IB"]
        elif self.year == 2014:
            regions = ["CT", "IB"]
        elif self.year == 2015:
            regions = ["CT"]
        elif self.year == 2016:
            regions = ["AN", "AR", "AS", "CB", "CE", "CL", "CM", "CT", "EX", "IB", "MC", "MD", "ML", "NC", "VC"]
        elif self.year == 2017:
            regions = ["CT"]
        elif self.year == 2018:
            regions = ["CL"]
        elif self.year == 2019:
            regions = ["CT", "IB"]
        elif self.year == 2020:
            regions = ["CT", "IB"]
        elif self.year == 2022:
            regions = ["CT"]
        elif self.year == 2023:
            regions = ["CT"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            SmartDayArrow(self.year, 12, 26),
            "San Esteban",
            "NRF" if regions == [""] else "RF"
        ) for region in regions]

    def holiday_jueves_santo(self):
        if self.year in [2011, 2016, 2017, 2022]:
            regions = ["AN", "AR", "AS", "CB", "CE", "CL", "CM", "CN", "EX", "GA", "IB", "MC", "MD", "ML", "NC", "PV", "RI", "VC"]
        elif self.year in [2012, 2013, 2014, 2015, 2019, 2020, 2021]:
            regions = ["AN", "AR", "AS", "CB", "CE", "CL", "CM", "CN", "EX", "GA", "IB", "MC", "MD", "ML", "NC", "PV", "RI"]
        elif self.year in [2018]:
            regions = ["AN", "AR", "AS", "CE", "CL", "CM", "CN", "EX", "GA", "IB", "MC", "MD", "ML", "NC", "PV", "RI"]
        elif self.year in [2023]:
            regions = ["AN", "AR", "AS", "CB", "CE", "CL", "CM", "CN", "EX", "GA", "IB", "MC", "MD", "ML", "NC", "PV", "VC"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            easter(self.year, self.easter_type).shift(days=-3),
            "Jueves Santo",
            "NRV" if regions == [""] else "RV"
        ) for region in regions]

    def holiday_lunes_de_pascua(self):
        if self.year == 2011:
            regions = ["CT", "IB", "NC", "PV", "RI", "VC"]
        elif self.year == 2012:
            regions = ["CT", "IB", "NC", "VC"]
        elif self.year == 2013:
            regions = ["CB", "CT", "IB", "NC", "PV", "RI", "VC"]
        elif self.year == 2014:
            regions = ["CM", "CT", "NC", "PV", "RI", "VC"]
        elif self.year == 2015:
            regions = ["CB", "CM", "CT", "IB", "NC", "PV", "RI", "VC"]
        elif self.year == 2016:
            regions = ["CT", "IB", "NC", "PV", "RI", "VC"]
        elif self.year == 2017:
            regions = ["CT", "IB", "NC", "PV", "RI", "VC"]
        elif self.year == 2018:
            regions = ["CL", "IB", "NC", "PV", "VC"]
        elif self.year == 2019:
            regions = ["CB", "CM", "CT", "IB", "NC", "PV", "RI", "VC"]
        elif self.year == 2020:
            regions = ["CB", "CM", "CT", "IB", "NC", "PV", "RI", "VC"]
        elif self.year == 2021:
            regions = ["CT", "IB", "NC", "PV", "RI", "VC"]
        elif self.year == 2022:
            regions = ["CN", "CT", "IB", "NC", "PV", "RI", "VC"]
        elif self.year == 2023:
            regions = ["CT", "IB", "MC", "NC", "PV", "RI"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            easter(self.year, self.easter_type).shift_to_weekday("monday", including=True),
            "Lunes de Pascua",
            "NRV" if regions == [""] else "RV"
        ) for region in regions]

    def holiday_lunes_de_pascua_granada(self):
        if self.year == 2011:
            date = SmartDayArrow(self.year, 6, 13)
        elif self.year == 2016:
            date = SmartDayArrow(self.year, 5, 16)
        elif self.year == 2022:
            date = SmartDayArrow(self.year, 6, 6)
        else:
            return []

        return [Holiday(
            self.locale,
            "CT",
            date,
            "Lunes de Pascua Granada",
            "F"
        )]

    def holiday_corpus_christi(self):
        if self.year in [2012, 2013, 2016, 2017, 2019, 2020, 2021, 2022, 2023]:
            regions = ["CM"]
        elif self.year in [2011, 2014, 2015]:
            regions = ["CM", "MD"]
        else:
            return []

        return [Holiday(
            self.locale,
            region,
            easter(self.year, self.easter_type).shift(days=60),
            "Corpus Christi",
            "RV"
        ) for region in regions]

    def holiday_eid_fitr(self):
        if self.year == 2022:
            date = SmartDayArrow(self.year, 5, 3)
        elif self.year == 2023:
            date = SmartDayArrow(self.year, 4, 21)
        else:
            return []

        return [Holiday(
            self.locale,
            "ML",
            date,
            "Fiesta del Eid Fitr",
            "RV"
        )]

    def holiday_eidul_adha(self):
        if self.year == 2012:
            date = SmartDayArrow(self.year, 10, 27)
        elif self.year == 2013:
            date = SmartDayArrow(self.year, 10, 15)
        elif self.year == 2014:
            date = SmartDayArrow(self.year, 10, 6)
        elif self.year == 2015:
            date = SmartDayArrow(self.year, 9, 25)
        elif self.year == 2016:
            date = SmartDayArrow(self.year, 9, 12)
        elif self.year == 2017:
            date = SmartDayArrow(self.year, 9, 1)
        elif self.year == 2018:
            date = SmartDayArrow(self.year, 8, 22)
        elif self.year == 2019:
            date = SmartDayArrow(self.year, 8, 12)
        elif self.year == 2020:
            date = SmartDayArrow(self.year, 7, 31)
        elif self.year == 2021:
            date = SmartDayArrow(self.year, 7, 20)
        elif self.year == 2022:
            date = SmartDayArrow(self.year, 7, 9)
        elif self.year == 2023:
            date = SmartDayArrow(self.year, 6, 29)
        else:
            return []

        return [Holiday(
            self.locale,
            "CE",
            date,
            "Fiesta del Sacrificio (Eidul Adha)",
            "RV"
        )]

    def holiday_aid_al_adha(self):
        if self.year == 2022:
            date = SmartDayArrow(self.year, 7, 11)
        elif self.year == 2023:
            date = SmartDayArrow(self.year, 6, 29)
        else:
            return []

        return [Holiday(
            self.locale,
            "ML",
            date,
            "Fiesta del Sacrificio (Aid Al Adha)",
            "RV"
        )]

    def holiday_lunes_siguiente_al_eidul_adha(self):
        if self.year == 2011:
            date = SmartDayArrow(self.year, 11, 7)
        else:
            return []

        return [Holiday(
            self.locale,
            "CE",
            date,
            "Lunes siguiente a la Fiesta del Sacrificio (Eidul Adha)",
            "RV"
        )]

    def holiday_aid_el_kebir(self):
        if self.year == 2011:
            date = SmartDayArrow(self.year, 11, 7)
        elif self.year == 2012:
            date = SmartDayArrow(self.year, 10, 26)
        elif self.year == 2013:
            date = SmartDayArrow(self.year, 10, 15)
        elif self.year == 2014:
            date = SmartDayArrow(self.year, 10, 4)
        elif self.year == 2015:
            date = SmartDayArrow(self.year, 9, 25)
        elif self.year == 2016:
            date = SmartDayArrow(self.year, 9, 12)
        elif self.year == 2017:
            date = SmartDayArrow(self.year, 9, 1)
        elif self.year == 2018:
            date = SmartDayArrow(self.year, 8, 22)
        elif self.year == 2019:
            date = SmartDayArrow(self.year, 8, 12)
        elif self.year == 2020:
            date = SmartDayArrow(self.year, 7, 31)
        elif self.year == 2021:
            date = SmartDayArrow(self.year, 7, 21)
        else:
            return []

        return [Holiday(
            self.locale,
            "ML",
            date,
            "Fiesta del Sacrificio (Aid El Kebir)",
            "RV"
        )]



# coding=utf-8
from dateutil.easter import EASTER_WESTERN

from holidata.utils import SmartDayArrow
from .holidays import Locale, Holiday

"""
sources: http://www.pisrs.si/Pis.web/pregledPredpisa?id=ZAKO865#
"""


class sl_SI(Locale):
    """
    01-01: [NF] Novo leto
    02-08: [NF] Prešernov dan
    04-27: [NF] Dan upora proti okupatorju
    05-01: [NF] Praznik dela
    05-02: [NF] Praznik dela
    06-25: [NF] Dan državnosti
    08-15: [NRF] Marijino vnebovzetje
    10-31: [NRF] Dan reformacije
    11-01: [NF] Dan spomina na mrtve
    12-25: [NF] Božič
    12-26: [NF] Dan samostojnosti in enotnosti
    Easter: [NRV] Velikonočna nedelja
    1 day after Easter: [NRV] Velikonočni ponedeljek
    50 days after Easter: [NRV] Binkošti
    """

    locale = "sl-SI"
    easter_type = EASTER_WESTERN

    def holiday_novo_leto(self):
        """
        From 1955 until May 2012, when the National Assembly of Slovenia passed the Public Finance Balance Act,
        2 January was a work-free day. It was reintroduced in 2017.
        2012<: https://www.uradni-list.si/1/objava.jsp?sop=2012-01-1700
        2016<: https://www.uradni-list.si/1/objava.jsp?sop=2016-01-3568
        """
        return [Holiday(
            self.locale,
            "",
            SmartDayArrow(self.year, 1, 2),
            "Novo leto",
            "NF"
        )] if self.year not in [2013, 2014, 2015, 2016] else []



# coding=utf-8
from dateutil.easter import EASTER_WESTERN

from .holidays import Locale


class de_CH(Locale):
    """
    01-01: [NF] Neujahrstag
    01-02: [BE,JU,TG,VD] [F] Berchtoldstag
    01-06: [SZ,TI,UR] [RF] Heilige Drei Könige
    03-19: [NW,SZ,TI,UR,VS] [RF] Josefstag
    05-01: [BL,BS,GR,NE,SH,TG,TI,ZH] [F] Tag der Arbeit
    08-01: [NF] Bundesfeier
    08-15: [AI] [RF] Mariä Himmelfahrt
    08-15: [JU,LU,NW,OW,SZ,TI,UR,VS,ZG] [RF] Mariä Himmelfahrt
    11-01: [AI,GL,JU,LU,NW,OW,SG,SZ,TI,UR,VS,ZG] [RF] Allerheiligen
    12-08: [AI] [RF] Mariä Empfängnis
    12-08: [LU,NW,OW,SZ,TI,UR,VS,ZG] [RF] Mariä Empfängnis
    12-25: [NRF] Weihnachtstag
    12-26: [AI,AR,BE,BL,BS,GL,GR,LU,SG,SH,SZ,TG,TI,UR,ZH] [RF] Stephanstag
    2 days before Easter: [AG,AI,AR,BE,BL,BS,FR,GE,GL,GR,JU,LU,NE,NW,OW,SG,SH,SO,SZ,TG,UR,VD,ZG,ZH] [RV] Karfreitag
    Easter: [NRV] Ostersonntag
    1 day after Easter: [AI,AR,BE,BL,BS,GE,GL,GR,JU,SG,SH,SZ,TG,TI,UR,VD,ZH] [RV] Ostermontag
    39 days after Easter: [NRV] Auffahrt
    49 days after Easter: [NRV] Pfingstsonntag
    50 days after Easter: [AI,AR,BE,BL,BS,GE,GL,GR,JU,SG,SH,SZ,TG,TI,UR,VD,ZH] [RV] Pfingstmontag
    60 days after Easter: [AI,JU,LU,NW,OW,SZ,TI,UR,VS,ZG] [RV] Fronleichnam
    """

    locale = "de-CH"
    easter_type = EASTER_WESTERN



# coding=utf-8
from dateutil.easter import EASTER_WESTERN

from holidata.utils import SmartDayArrow
from .holidays import Holiday, Locale


class nl_NL(Locale):
    """
    01-01: [NF] Nieuwjaarsdag
    05-04: [NF] Dodenherdenking
    05-05: [NF] Bevrijdingsdag
    12-05: [NRF] Sinterklaas
    12-25: [NRF] Eerste Kerstdag
    12-26: [NRF] Tweede Kerstdag
    2 days before Easter: [NRV] Goede Vrijdag
    Easter: [NRV] Eerste Paasdag
    1 day after Easter: [NRV] Tweede Paasdag
    39 days after Easter: [NRV] Hemelvaartsdag
    49 days after Easter: [NRV] Eerste Pinksterdag
    50 days after Easter: [NRV] Tweede Pinksterdag
    """

    locale = "nl-NL"
    easter_type = EASTER_WESTERN

    def holiday_koningsdag(self):
        """04-27 or saturday before if it falls on sunday: [NF] Koninginnedag/Koningsdag """
        if self.year < 2014:
            date = SmartDayArrow(self.year, 4, 30)
            description = "Koninginnedag"
        else:
            date = SmartDayArrow(self.year, 4, 27)
            description = "Koningsdag"

        if date.weekday() == "sunday":
            date = date.shift(days=-1)

        return [Holiday(
            locale=self.locale,
            region="",
            date=date,
            description=description,
            flags="NV",
            notes="")]

    def holiday_koninkrijksdag(self):
        date = SmartDayArrow(self.year, 12, 15)

        if date.weekday() == "sunday":
            date = date.shift(days=1)

        return [Holiday(
            locale=self.locale,
            region="",
            date=date,
            description="Koninkrijksdag",
            flags="NV",
            notes="")]



# coding=utf-8
from dateutil.easter import EASTER_WESTERN

from .holidays import Locale


class nl_BE(Locale):
    """
    01-01: [NF] Nieuwjaar
    05-01: [NF] Dag van de arbeid
    07-21: [NF] Nationale feestdag
    08-15: [NRF] Onze Lieve Vrouw hemelvaart
    11-01: [NRF] Allerheiligen
    11-11: [NF] Wapenstilstand
    12-25: [NRF] Kerstmis
    Easter: [NRV] Pasen
    1 day after Easter: [NRV] Paasmaandag
    39 days after Easter: [NRV] Onze Lieve Heer hemelvaart
    49 days after Easter: [NRV] Pinksteren
    50 days after Easter: [NRV] Pinkstermaandag
    """

    locale = "nl-BE"
    easter_type = EASTER_WESTERN



from dateutil.easter import EASTER_WESTERN

from holidata.utils import SmartDayArrow
from .holidays import Holiday, Locale


class is_IS(Locale):
    """
    01-01: [NRF] Nýársdagur
    05-01: [NF] Verkalýðsdagurinn
    06-17: [NF] Þjóðhátíðardagurinn
    12-25: [NRF] Jóladagur
    12-26: [NRF] Annar dagur jóla
    3 days before Easter: [NRV] Skírdagur
    2 days before Easter: [NRV] Föstudagurinn langi
    Easter: [NRV] Páskadagur
    1 day after Easter: [NRV] Annar dagur páska
    39 days after Easter: [NRV] Uppstigningardagur
    49 days after Easter: [NRV] Hvítasunnudagur
    50 days after Easter: [NRV] Annar dagur hvítasunnu
    1. monday in August: [NV] Frídagur verslunarmanna
    """

    locale = "is-IS"
    easter_type = EASTER_WESTERN

    def holiday_first_day_of_summer(self):
        """
        Calculate sumardagurinn fyrsti (first day of summer).

        The holiday falls on the first Thursday after the 18th of April.
        """
        return [
            Holiday(
                locale=self.locale,
                region="",
                date=SmartDayArrow(self.year, 4, 18).shift_to_weekday("thursday"),
                description="Sumardagurinn fyrsti",
                flags="NV",
                notes="",
            )
        ]

    def holiday_half_days(self):
        """
        Define half-day holidays.

        Both Christmas Eve (_aðfangadagur jóla_) and New Year's Eve
        (_gamlársdagur_) are public holidays in Iceland from 13:00 only.
        They're included as full-day holidays, but with an explanatory
        note.

        12-24: [NRF] Aðfangadagur jóla
        12-31: [NF] Gamlársdagur
        """
        return [
            Holiday(
                locale=self.locale,
                region="",
                date=SmartDayArrow(self.year, 12, 24),
                description="Aðfangadagur jóla",
                flags="NRF",
                notes="Holiday from 13:00",
            ),
            Holiday(
                locale=self.locale,
                region="",
                date=SmartDayArrow(self.year, 12, 31),
                description="Gamlársdagur",
                flags="NF",
                notes="Holiday from 13:00",
            ),
        ]



# coding=utf-8
from dateutil.easter import EASTER_WESTERN

from .holidays import Locale

"""
source: http://prawo.sejm.gov.pl/isap.nsf/download.xsp/WDU20150000090/O/D20150090.pdf
"""


class pl_PL(Locale):
    """
    01-01: [NF] Nowy Rok
    01-06: [NRF] Trzech Króli
    05-01: [NF] Święto Pracy
    05-03: [NF] Święto Konstytucji Trzeciego Maja
    08-15: [NRF] Wniebowzięcie Najświętszej Maryi Panny
    11-01: [NRF] Wszystkich Świętych
    11-11: [NF] Narodowe Święto Niepodległości
    12-25: [NRF] Boże Narodzenie (pierwszy dzień)
    12-26: [NRF] Boże Narodzenie (drugi dzień)
    Easter: [NRV] Wielkanoc
    1 day after Easter: [NRV] Poniedziałek Wielkanocny
    49 days after Easter: [NRV] Zielone Świątki
    60 days after Easter: [NRV] Boże Ciało
    """

    locale = "pl-PL"
    easter_type = EASTER_WESTERN



# coding=utf-8
from dateutil.easter import EASTER_WESTERN

from holidata.utils import SmartDayArrow
from .holidays import Locale, Holiday


class de_DE(Locale):
    """
    01-01: [NF] Neujahr
    01-06: [BW,BY,ST] [RF] Heilige drei Könige
    05-01: [NF] Erster Maifeiertag
    08-15: [SL] [RF] Mariä Himmelfahrt
    10-03: [NRF] Tag der Deutschen Einheit
    11-01: [BW,BY,NW,RP,SL] [RF] Allerheiligen
    12-24: [NRF] Heilig Abend
    12-25: [NRF] Weihnachtstag
    12-26: [NRF] Zweiter Weihnachtstag
    12-31: [NF] Silvester
    2 days before Easter: [NRV] Karfreitag
    Easter: [NRV] Ostern
    1 day after Easter: [NRV] Ostermontag
    39 days after Easter: [NRV] Christi Himmelfahrt
    49 days after Easter: [NRV] Pfingstsonntag
    50 days after Easter: [NRV] Pfingstmontag
    60 days after Easter: [BW,BY,HE,NW,RP,SL] [RV] Fronleichnam
    """

    locale = "de-DE"
    easter_type = EASTER_WESTERN

    def holiday_buss_und_bettag(self):
        """11 days before 4. sunday before 12-25: [SN] [RV] Buß- und Bettag"""

        return [Holiday(
            self.locale,
            "SN",
            SmartDayArrow(self.year, 12, 25).shift_to_weekday("sunday", order=4, reverse=True).shift(days=-11),
            "Buß- und Bettag",
            "RV"
        )]

    def holiday_reformationstag(self):
        """
        before 2018: 10-31: [BB, MV, SN, ST, TH] [RF] Reformationstag
        since 2018:  10-31: [BB, BH, HH, MV, NI, SH, SN, ST, TH] [RF] Reformationstag
        2017:        10-31: [NRF] Reformationstag (national holiday because of 500th anniversary)

        """
        if self.year == 2017:
            regions = [""]
        elif self.year < 2018:
            regions = ["BB", "MV", "SN", "ST", "TH"]
        else:
            regions = ["BB", "BH", "HH", "MV", "NI", "SH", "SN", "ST", "TH"]

        return [Holiday(
            self.locale,
            region,
            SmartDayArrow(self.year, 10, 31),
            "Reformationstag",
            "NRF" if regions == [""] else "RF"
        ) for region in regions]

    def holiday_frauentag(self):
        """
        03-08: [BE] [F] Frauentag

        Introduced 2019 for Berlin
        http://gesetze.berlin.de/jportal/?quelle=jlink&query=FeiertG+BE+%C2%A7+1&psml=bsbeprod.psml&max=true
        """
        if self.year >= 2019:
            return [Holiday(
                self.locale,
                "BE",
                SmartDayArrow(self.year, 3, 8),
                "Internationaler Frauentag",
                "F"
            )]
        else:
            return []

    def holiday_tag_der_befreiung(self):
        """
        2020-05-08: [BE] [F] 75. Jahrestag der Befreiung vom Nationalsozialismus und der Beendigung des Zweiten Weltkrieges in Europa

        Introduced 2019 for Berlin
        http://gesetze.berlin.de/jportal/?quelle=jlink&query=FeiertG+BE+%C2%A7+1&psml=bsbeprod.psml&max=true
        """
        if self.year == 2020:
            return [Holiday(
                self.locale,
                "BE",
                SmartDayArrow(self.year, 5, 8),
                "75. Jahrestag der Befreiung vom Nationalsozialismus und der Beendigung des Zweiten Weltkrieges in Europa",
                "F"
            )]
        else:
            return []



from .holidays import *
from .emitters import Emitter

def get_country_for(identifier):
    country_class = Country.get(identifier)

    if not country_class:
        raise ValueError("No plugin found for country id '{}'!".format(identifier))

    return country_class()


def get_locale_class_for(identifier):
    locale_class = Locale.get(identifier)

    if not locale_class:
        raise ValueError("No plugin found for locale: {}!".format(identifier))

    return locale_class


def create_locale_for(country_id=None, lang_id=None, year=None):
    country_class = get_country_for(country_id)

    if lang_id is not None and lang_id.lower() not in country_class.languages:
        raise ValueError("Language '{}' is not defined for country '{}'!".format(lang_id, country_class.id))
    elif lang_id is None and country_class.default_lang is not None:
        lang_id = country_class.default_lang
    elif lang_id is None:
        raise ValueError("Country '{}' has no default language specified! Please choose one of [{}].".format(country_id, ", ".join(country_class.languages)))

    locale_class = get_locale_class_for("{}-{}".format(lang_id, country_id))

    return locale_class(year)


def create_emitter_for(identifier):
    emitter_class = Emitter.get(identifier)

    if not emitter_class:
        raise ValueError("Unsupported output format: '{}'!".format(identifier))

    return emitter_class()


def parse_year(year):
    try:
        return int(year)
    except:
        raise ValueError(f"Invalid year '{year}'! Has to be an integer.")


class Holidata:
    locale = None
    emitter = None

    def __init__(self, country=None, language=None, year=None, output=None):
        self.locale = create_locale_for(country_id=country, lang_id=language, year=parse_year(year))
        self.emitter = create_emitter_for(output)

    def __str__(self):
        return self.emitter.output(self.locale)



# coding=utf-8
from dateutil.easter import EASTER_WESTERN

from holidata.utils import SmartDayArrow
from .holidays import Locale, Holiday

"""
source: https://www.riksdagen.se/sv/dokument-lagar/dokument/svensk-forfattningssamling/lag-1989253-om-allmanna-helgdagar_sfs-1989-253
source: https://www.riksdagen.se/sv/dokument-lagar/dokument/svensk-forfattningssamling/semesterlag-1977480_sfs-1977-480
"""


class sv_SE(Locale):
    """
    01-01: [NF] Nyårsdagen
    01-06: [NRF] Trettondedag jul
    05-01: [NF] Första maj
    06-06: [NF] Nationaldagen
    12-24: [NRF] Julafton
    12-25: [NRF] Juldagen
    12-26: [NRF] Annandag jul
    12-31: [NF] Nyårsafton
    2 days before Easter: [NRV] Långfredagen
    Easter: [NRV] Påskdagen
    1 day after Easter: [NRV] Annandag påsk
    39 days after Easter: [NRV] Kristi himmelsfärdsdag
    49 days after Easter: [NRV] Pingstdagen
    """

    locale = "sv-SE"
    easter_type = EASTER_WESTERN

    def __midsommar(self):
        """
        Find the Saturday between 20 and 26 June
        """
        return SmartDayArrow(self.year, 6, 19).shift_to_weekday("saturday", order=1, reverse=False)

    def holiday_midsommarafton(self):
        """
        The day before midsommardagen: [NV] Midsommarafton
        """
        return [Holiday(
            self.locale,
            "",
            self.__midsommar().shift(days=-1),
            "Midsommarafton",
            "NV"
        )]

    def holiday_midsommardagen(self):
        """
        Saturday between 20 and 26 June: [NV] Midsommardagen
        """
        return [Holiday(
            self.locale,
            "",
            self.__midsommar(),
            "Midsommardagen",
            "NV"
        )]

    def holiday_alla_helgons_dag(self):
        """
        Saturday between 31 October and 6 November: [NRV] Alla helgons dag
        """
        return [Holiday(
            self.locale,
            "",
            SmartDayArrow(self.year, 10, 30).shift_to_weekday("saturday", order=1, reverse=False),
            "Alla helgons dag",
            "NRV"
        )]
